//=============================================================================
// HitSounds
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Accumulates damage for hit sound pitch calculation and plays hit sounds.
//=============================================================================


class HitSounds extends ReplicationInfo;


//=============================================================================
// Imports
//=============================================================================

#exec audio import file=Sounds\HitSoundEnemy1.wav      name=HitSoundEnemy1      group=HitSounds
#exec audio import file=Sounds\HitSoundEnemy2.wav      name=HitSoundEnemy2      group=HitSounds
#exec audio import file=Sounds\HitSoundEnemyTTM.wav    name=HitSoundEnemyTTM    group=HitSounds
#exec audio import file=Sounds\HitSoundEnemyQ3.wav     name=HitSoundEnemyQ3     group=HitSounds
#exec audio import file=Sounds\HitSoundEnemyUTPure.wav name=HitSoundEnemyUTPure group=HitSounds
#exec audio import file=Sounds\HitSoundProtected.wav   name=HitSoundProtected   group=HitSounds
#exec audio import file=Sounds\HitSoundTeam.wav        name=HitSoundTeam        group=HitSounds


//=============================================================================
// Variables
//=============================================================================

var() array<Sound> HitSoundEnemy;
var() array<float> HitSoundDefaultPitch;
var() Sound HitSoundProtected;
var() Sound HitSoundTeam;

var float AccumulatedDamage;            // total enemy damage dealt out in the last time interval
var bool bProtectedHit;                 // indicates a hit when the victim had respawn protection
var bool bTeamHit;                      // indicates a team hit in the last time interval

var array<HitSounds> HitSoundsList;     // global list of all HitSounds actors

var PlayerController MyOwner;


//=============================================================================
// Replication
//=============================================================================

replication
{
  reliable if ( Role == ROLE_Authority )
    MyOwner;
  
  unreliable if ( Role == ROLE_Authority )
    ClientPlayHitSound;
}


//=============================================================================
// PostBeginPlay
//
// Initializes the HitSounds actor and adds it to the global list.
//=============================================================================

function PostBeginPlay()
{
  SetTimer(0.1, True);
  default.HitSoundsList[default.HitSoundsList.Length] = Self;
  
  MyOwner = PlayerController(Owner);
}


//=============================================================================
// Destroyed
//
// Removes the HitSounds actor from the global list.
//=============================================================================

function Destroyed()
{
  local int i;
  
  for (i = default.HitSoundsList.Length; i >= 0; i--)
    if ( default.HitSoundsList[i] == None || default.HitSoundsList[i] == Self )
      default.HitSoundsList.Remove(i, 1);
}


//=============================================================================
// Timer
//
// Calculates a hit sound pitch from the accumulated damage and plays a hit
// sound if any damage was dealt out.
//=============================================================================

function Timer()
{
  local float HitSoundPitch;
  
  if ( AccumulatedDamage > 0 )
    HitSoundPitch = FClamp(1.7 - LogE(AccumulatedDamage) / LogE(50), 0.2, 1.5);
  
  if ( class'TournamentServerSettings'.default.bAllowHitSounds && (HitSoundPitch > 0 || bTeamHit || bProtectedHit) ) {
    if ( !class'TournamentServerSettings'.default.bAllowVariableHitSounds && HitSoundPitch > 0 )
      HitSoundPitch = -1;
    ClientPlayHitSound(HitSoundPitch, bTeamHit, bProtectedHit);
  }
  
  AccumulatedDamage = 0;
  bTeamHit = False;
  bProtectedHit = False;
}


//=============================================================================
// ClientPlayHitSound
//
// Plays a hit sound on the client.
//=============================================================================

simulated function ClientPlayHitSound(float Pitch, bool bSameTeam, bool bProtected)
{
  if ( class'TournamentClientSettings'.default.HitSoundVolume <= 0 )
    return;
  else if ( !class'TournamentClientSettings'.default.bVariableHitSounds && Pitch != 0 || Pitch < 0 )
    Pitch = HitSoundDefaultPitch[class'TournamentClientSettings'.default.HitSound % HitSoundDefaultPitch.Length];
  
  if ( MyOwner.ViewTarget != None ) {
    if ( bSameTeam )
	  	MyOwner.ViewTarget.PlaySound(HitSoundTeam,,
	  	    class'TournamentClientSettings'.default.HitSoundVolume,,,, False);
    if ( bProtected )
	  	MyOwner.ViewTarget.PlaySound(HitSoundProtected,,
	  	    class'TournamentClientSettings'.default.HitSoundVolume,,,, False);
    if ( Pitch > 0 )
      MyOwner.ViewTarget.PlaySound(
          HitSoundEnemy[class'TournamentClientSettings'.default.HitSound % HitSoundEnemy.Length],,
          class'TournamentClientSettings'.default.HitSoundVolume,,, Pitch, False);
  }
  if ( MyOwner.DemoViewer != None && MyOwner.DemoViewer.ViewTarget != None ) {
    if ( bSameTeam )
	  	MyOwner.DemoViewer.ViewTarget.PlaySound(HitSoundTeam,,
	  	    class'TournamentClientSettings'.default.HitSoundVolume,,,, False);
    if ( bProtected )
	  	MyOwner.DemoViewer.ViewTarget.PlaySound(HitSoundProtected,,
	  	    class'TournamentClientSettings'.default.HitSoundVolume,,,, False);
    if ( Pitch > 0 )
      MyOwner.DemoViewer.ViewTarget.PlaySound(
          HitSoundEnemy[class'TournamentClientSettings'.default.HitSound % HitSoundEnemy.Length],,
          class'TournamentClientSettings'.default.HitSoundVolume,,, Pitch, False);
  }
}


//=============================================================================
// AddDamage
//
// Accumulates the damage dealt out by the player.
//=============================================================================

function AddDamage(int Damage, Pawn Victim)
{
  if ( Damage == 0 || Victim == None || Victim.Controller == Owner || !MyOwner.LineOfSightTo(Victim) )
    return;
  
  if ( MyOwner.SameTeamAs(Victim.Controller) )
    bTeamHit = True;
  else if ( Level.TimeSeconds - Victim.SpawnTime < DeathMatch(Level.Game).SpawnProtectionTime )
    bProtectedHit = True;
  else
    AccumulatedDamage += Damage;
}


//=============================================================================
// CalcHitSoundsFor
//
// Plays a hit sound on the client.
//=============================================================================

static function CalcHitSoundsFor(PlayerController Player, int Damage, Pawn Victim)
{
  local HitSounds FoundHitSounds;
  local int i;
  
  if ( Player == None || Damage <= 0 || Victim == None )
    return;
  
  for (i = 0; i < default.HitSoundsList.Length; i++) {
    if ( default.HitSoundsList[i] == None ) {
      default.HitSoundsList.Remove(i, 1);
      i--;
    }
    else if ( default.HitSoundsList[i].Owner == Player ) {
      FoundHitSounds = default.HitSoundsList[i];
      break;
    }
    else if ( default.HitSoundsList[i].Owner == None )
      default.HitSoundsList[i].Destroy();
  }
  
  if ( FoundHitSounds == None )
    FoundHitSounds = Player.Spawn(class'HitSounds', Player);
  if ( FoundHitSounds != None )
    FoundHitSounds.AddDamage(Damage, Victim);
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  HitSoundEnemy=(Sound'HitSoundEnemy1',Sound'HitSoundEnemy2',Sound'HitSoundEnemyTTM',Sound'HitSoundEnemyQ3',Sound'HitSoundEnemyUTPure')
  HitSoundDefaultPitch=(1.0,1.0,1.2,1.0,1.0)
  HitSoundProtected=Sound'HitSoundProtected'
  HitSoundTeam=Sound'HitSoundTeam'
  bReplicateMovement=False
}