//=============================================================================
// TournamentPlayer
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// PlayerController for Tournament game types.
//=============================================================================


class TournamentPlayer extends xPlayer
    dependson(TournamentUtil);


//=============================================================================
// Constants
//=============================================================================

// GUIQuestionPage button codes
const QBTN_Ok = 1;


//=============================================================================
// Replication
//=============================================================================

replication
{
  reliable if ( Role == ROLE_Authority )
    ClientSendServerSettings, ClientReset;
  
  reliable if ( Role < ROLE_Authority )
    ServerRequestServerSettings, ServerSetServerSettings, ServerSetOldVersion, Ready;
  
  unreliable if ( Role < ROLE_Authority )
    ServerDelayedAnnouncement;
}


//=============================================================================
// PreBeginPlay
//
// Force garbage collection.
//=============================================================================

simulated function PreBeginPlay()
{
  if ( Level.NetMode == NM_Client )
    ConsoleCommand("OBJ GARBAGE");
  Super.PreBeginPlay();
}


//=============================================================================
// PostBeginPlay
//
// Set spectating FOV.
//=============================================================================

simulated function PostBeginPlay()
{
  Super.PostBeginPlay();
  default.FOVAngle = DefaultFOV;
  bHideSpectatorBeacons = class'TournamentClientSettings'.default.bHideSpectatorBeacons;
}


//=============================================================================
// ClientSetHUD
//
// Check for the correct client version. (older than 2225 isn't suported)
//=============================================================================

simulated function ClientSetHUD(class<HUD> newHUDClass, class<Scoreboard> newScoringClass)
{
  if ( int(Level.EngineVersion) < 2225 ) {
    ServerSetOldVersion();
    newHUDClass = class'OldVersionHUD';
    newScoringClass = class'ScoreboardDeathMatch';
  }
  Super.ClientSetHUD(newHUDClass, newScoringClass);
  FOV(DefaultFOV);
}


//=============================================================================
// ClientReset
//
// Reset clientside stuff.
//=============================================================================

simulated function ClientReset()
{
  if ( MyHud != None )
    MyHud.Reset();
}


//=============================================================================
// Reset
//
// Reset clientside stuff.
//=============================================================================

function Reset()
{
  ClientReset();
  Super.Reset();
}


//=============================================================================
// ServerSetOldVersion
//
// Puts the player in spectator mode.
//=============================================================================

function ServerSetOldVersion()
{
  PlayerReplicationInfo.bOnlySpectator = True;
  PlayerReplicationInfo.bIsSpectator   = True;
  PlayerReplicationInfo.bOutOfLives    = True;
  Level.Game.NumSpectators++;
  Level.Game.NumPlayers--;
  GotoState('Auto');
}


//=============================================================================
// ServerDoCombo
//
// Executes an adrenaline combo serverside if this combo isn't disabled.
//=============================================================================

function ServerDoCombo(class<Combo> ComboClass)
{
  if ( ComboIsAllowed(ComboClass) )
    Super.ServerDoCombo(ComboClass);
}


//=============================================================================
// ComboIsAllowed
//
// Returns whether an adrenaline combo may be used.
//=============================================================================

function bool ComboIsAllowed(class<Combo> ComboClass)
{
  switch (ComboClass) {
    case class'ComboBerserk':
      return class'TournamentServerSettings'.default.bAllowComboBerserk;
    case class'ComboDefensive':
      return class'TournamentServerSettings'.default.bAllowComboBooster;
    case class'ComboInvis':
      return class'TournamentServerSettings'.default.bAllowComboInvisible;
    case class'ComboSpeed':
      return class'TournamentServerSettings'.default.bAllowComboSpeed;
    default:
      return true;
  }
}


//=============================================================================
// PlayStartupMessage
//
// Display the current match status.
//=============================================================================

function PlayStartupMessage(byte StartupStage)
{
  ReceiveLocalizedMessage(class'TournamentStartupMessage', StartupStage, PlayerReplicationInfo);
}


//=============================================================================
// exec Ready
//
// Toggle 'ready' status.
//=============================================================================

exec function Ready()
{
  PlayerReplicationInfo.bReadyToPlay = !PlayerReplicationInfo.bReadyToPlay;
}


//=============================================================================
// exec Ver
//
// Displays the current version of Tournament and (if loaded) Anti TCC.
//=============================================================================

exec function Ver()
{
  ClientMessage(class'TournamentUtil'.static.GetTournamentVersion());
  Mutate("AntiTCC Version");
}


//=============================================================================
// ShowIDs
//
// Displays the current version of Anti TCC.
//=============================================================================

exec function ShowIDs()
{
  Mutate("AntiTCC ShowIDs");
}


//=============================================================================
// exec Whois
//
// Displays other names a player has used on the server.
//=============================================================================

exec function Whois(string PlayerName)
{
  Mutate("AntiTCC Whois"@PlayerName);
}


//=============================================================================
// ChangeLoadout
//
// Disable the ChangeLoadout console command.
//=============================================================================

exec function ChangeLoadout(string Loadout)
{
  Mutate("AntiTCCcl"@Loadout);
}


//=============================================================================
// JoeMenuTest
//
// Disable the JoeMenuTest console command.
//=============================================================================

exec function JoeMenuTest(string NewMenu)
{
  Mutate("AntiTCCjmt"@NewMenu);
}


//=============================================================================
// CreateCameraEffect
//
// Disable the CreateCameraEffect console command.
//=============================================================================

exec function CreateCameraEffect(class<CameraEffect> EffectClass)
{
  Mutate("AntiTCCcce"@EffectClass);
}


//=============================================================================
// PlayAnnouncement
//
// Play announcements also for demo spectator.
//=============================================================================

simulated function PlayAnnouncement(sound ASound, byte AnnouncementLevel, optional bool bForce)
{
  local float Atten;
  
  if ( AnnouncementLevel > AnnouncerLevel )
    return;
  if ( !bForce && Level.TimeSeconds - LastPlaySound < 1 )
    return;
  LastPlaySound = Level.TimeSeconds;  // so voice messages won't overlap
  LastPlaySpeech = Level.TimeSeconds;   // don't want chatter to overlap announcements
  
  Atten = FClamp(0.1 + float(AnnouncerVolume) * 0.225, 0.2, 1.0);
  ASound = CustomizeAnnouncer(ASound);
  ClientPlaySound(ASound, true, Atten, SLOT_Talk);
  if ( DemoViewer != None )
    DemoViewer.ClientPlaySound(ASound, true, Atten, SLOT_Talk);
}


//=============================================================================
// AwardAdrenaline
//
// Play announcements in server demos.
//=============================================================================

function AwardAdrenaline(float Amount)
{
  if ( bAdrenalineEnabled ) {
    if ( Adrenaline < AdrenalineMax && Adrenaline + Amount >= AdrenalineMax
        && (Pawn == None || !Pawn.InCurrentCombo()) ) {
      DelayedAnnouncement(sound'AnnouncerMain.Adrenalin', 15);
    }
    Super(PlayerController).AwardAdrenaline(Amount);
  }
}


//=============================================================================
// ServerDelayedAnnouncement
//
// Replicates a DelayedAnnouncement to the server.
//=============================================================================

function ServerDelayedAnnouncement(Sound AnnouncementSound, byte Delay)
{
  DelayedAnnouncement(AnnouncementSound, Delay);
}


//=============================================================================
// DelayedAnnouncement
//
// Delayed announcement similar to ClientDelayedAnnouncement but on the server
// for fixing server demos.
//=============================================================================

simulated function DelayedAnnouncement(Sound AnnouncementSound, byte Delay)
{
  local DelayedAnnouncement A;
  
  if ( PlayerReplicationInfo.Owner == None )
    PlayerReplicationInfo.SetOwner(Self);
  
  A = Spawn(class'DelayedAnnouncement', PlayerReplicationInfo);
  A.AnnounceSound = AnnouncementSound;
  A.SetTimer(0.1 * Delay, false);
}


//=============================================================================
// ClientDelayedAnnouncement
//
// Catch combo whore, flak monkey and head hunter announcements.
// *cough* not hacky at all ;-)
//=============================================================================

function ClientDelayedAnnouncement(sound AnnouncementSound, byte Delay)
{
  local TournamentGameReplicationInfo TGRI;
  
  DelayedAnnouncement(AnnouncementSound, Delay);
  
  if ( TournamentHUD(myHud) != None && GameReplicationInfo != None ) {
    TGRI = TournamentGameReplicationInfo(GameReplicationInfo);
    switch (AnnouncementSound) {
      case sound'FlackMonkey':
        TournamentHUD(myHud).FlakMonkeyTime = GameReplicationInfo.ElapsedTime - TGRI.GameStartTime;
        break;
      case sound'ComboWhore':
        TournamentHUD(myHud).ComboWhoreTime = GameReplicationInfo.ElapsedTime - TGRI.GameStartTime;
        break;
      case sound'HeadHunter':
        TournamentHUD(myHud).HeadHunterTime = GameReplicationInfo.ElapsedTime - TGRI.GameStartTime;
        break;
    }
  }
}


//=============================================================================
// exec ShowMenu
//
// Shows the mid-game menu.
//=============================================================================

exec function ShowMenu()
{
  // Pause if not already
  if ( Level.NetMode == NM_StandAlone && Level.Pauser == None )
    SetPause(true);
  
  StopForceFeedback();
  ClientOpenMenu(string(class'TGUIMidGameMenu'));
}


//=============================================================================
// exec AdminMenu
//
// Shows the admin menu.
//=============================================================================

exec function AdminMenu(string CommandLine)
{
  local string MenuClass;
  
  StopForceFeedback();
  
  if ( TournamentHUD(MyHud) != None && TournamentHUD(MyHud).ConfigMenuClass != "" )
    MenuClass = TournamentHUD(MyHud).ConfigMenuClass;
  else
    MenuClass = string(class'TGUISettingsMenu');
  
  // Pause if not already
  if ( Level.NetMode == NM_StandAlone && Level.Pauser == None ) {
    SetPause(true);
    ClientOpenMenu(MenuClass,, "SetPause" @ CommandLine, "Admin");
  }
  else
    ClientOpenMenu(MenuClass,, CommandLine, "Admin");
}


//=============================================================================
// exec Say
//
// Send a message to all players. Parse % variables like in TeamSay messages.
//=============================================================================

exec function Say(string Msg)
{
  Super.Say(Level.Game.ParseMessageString(Level.Game.BaseMutator, Self, Msg));
}


//=============================================================================
// state PlayerWaiting
//
// Show the scoreboard shortly after dying.
//=============================================================================

state PlayerWaiting
{
  function ServerRestartPlayer()
  {
    if ( Level.TimeSeconds < WaitDelay )
      return;
    if ( Level.NetMode == NM_Client )
      return;
    if ( Level.NetMode == NM_Standalone && Level.Game.bWaitingToStartMatch )
      PlayerReplicationInfo.bReadyToPlay = true;
    else
      Level.Game.RestartPlayer(Self);
  }
}


//=============================================================================
// state Dead
//
// Show the scoreboard shortly after dying.
//=============================================================================

state Dead
{
Begin:
  if ( GameReplicationInfo.MaxLives == 0 )
    Sleep(1.0);
  while (ViewTarget != None && ViewTarget != self && (Pawn(ViewTarget) == None || Pawn(ViewTarget).Health > 0))
    Sleep(1.0);
  myHUD.bShowScoreBoard = true;
}


//=============================================================================
// state GameEnded
//
// End client demo recording and take a screenshot of the scores.
//=============================================================================

state GameEnded
{
Begin:
  Sleep(0.0);
  if ( Viewport(Player) != None ) { // only for the local player
    if ( MyHud != None ) {
      if ( class'TournamentClientSettings'.default.bAutoScreenshot ) {
        MyHud.bShowDebugInfo = false;
        MyHud.bShowLocalStats = false;
        MyHud.bShowScoreBoard = true;
        Sleep(0.0); // wait for the next frame so the scoreboard can be drawn
        ConsoleCommand("SHOT");
      }
      else {
        Sleep(1.0);
        MyHud.bShowDebugInfo = false;
        MyHud.bShowLocalStats = false;
        MyHud.bShowScoreBoard = true;
      }
    }
    Sleep(3.0);
    ConsoleCommand("STOPDEMO");
  }
}


//=============================================================================
// ToggleZoom
//
// Toggles between zoom in and zoom out.
//=============================================================================

function ToggleZoom()
{
  if ( DefaultFOV != DesiredFOV )
    EndZoom();
  else
    StartZoom();
}


//=============================================================================
// StartZoom
//
// Zooms in.
//=============================================================================

function StartZoom()
{
  ZoomLevel = 0.0;
  bZooming = true;
}


//=============================================================================
// StopZoom
//
// Stops zooming in.
//=============================================================================

function StopZoom()
{
  bZooming = false;
}


//=============================================================================
// EndZoom
//
// Zooms out.
//=============================================================================

function EndZoom()
{
  local float MaxFOV, F;
  
  MaxFOV = 170;
  if ( TournamentGameReplicationInfo(GameReplicationInfo) != None
      && TournamentGameReplicationInfo(GameReplicationInfo).MaxFOV > 0 )
    MaxFOV = FClamp(TournamentGameReplicationInfo(GameReplicationInfo).MaxFOV, 80, 170);
  if ( Level.Netmode != NM_StandAlone )
    F = FClamp(DefaultFOV, 80, MaxFOV);
  else
    F = FClamp(DefaultFOV, 1, MaxFOV);
  
  bZooming = false;
  DesiredFOV = F;
}


//=============================================================================
// FixFOV
//
// Resets FOV angle to the default value.
//=============================================================================

function FixFOV()
{
  local float MaxFOV, F;
  
  MaxFOV = 170;
  if ( TournamentGameReplicationInfo(GameReplicationInfo) != None
      && TournamentGameReplicationInfo(GameReplicationInfo).MaxFOV > 0 )
    MaxFOV = FClamp(TournamentGameReplicationInfo(GameReplicationInfo).MaxFOV, 80, 170);
  if ( Level.Netmode != NM_StandAlone )
    F = FClamp(Default.DefaultFOV, 80, MaxFOV);
  else
    F = FClamp(Default.DefaultFOV, 1, MaxFOV);
  
  FOVAngle = F;
  DesiredFOV = F;
  DefaultFOV = F;
}


//=============================================================================
// SetFOV
//
// Sets the FOV angle.
//=============================================================================

function SetFOV(float NewFOV)
{
  DesiredFOV = NewFOV;
  FOVAngle = NewFOV;
}


//=============================================================================
// ResetFOV
//
// Resets the FOV angle.
//=============================================================================

function ResetFOV()
{
  local float MaxFOV, F;
  
  MaxFOV = 170;
  if ( TournamentGameReplicationInfo(GameReplicationInfo) != None
      && TournamentGameReplicationInfo(GameReplicationInfo).MaxFOV > 0 )
    MaxFOV = FClamp(TournamentGameReplicationInfo(GameReplicationInfo).MaxFOV, 80, 170);
  if ( Level.Netmode != NM_StandAlone )
    F = FClamp(DefaultFOV, 80, MaxFOV);
  else
    F = FClamp(DefaultFOV, 1, MaxFOV);
  
  DesiredFOV = F;
  FOVAngle = F;
}


//=============================================================================
// exec FOV
//
// Sets the FOV.
//=============================================================================

exec function FOV(float F)
{
  local float MaxFOV;
  
  MaxFOV = 170;
  if ( TournamentGameReplicationInfo(GameReplicationInfo) != None
      && TournamentGameReplicationInfo(GameReplicationInfo).MaxFOV > 0 )
    MaxFOV = FClamp(TournamentGameReplicationInfo(GameReplicationInfo).MaxFOV, 80, 170);
  if ( Level.Netmode != NM_StandAlone )
    F = FClamp(F, 80, MaxFOV);
  else
    F = FClamp(F, 1, MaxFOV);
  DefaultFOV = F;
  DesiredFOV = F;
  default.FOVAngle = F;
  if ( Pawn == None )
    FOVAngle = F;
  SaveConfig();
}


//=============================================================================
// delegate OnClientReceivedSettings
//
// Called on the client when a server property value is received.
//=============================================================================

delegate OnClientReceivedSettings(TournamentUtil.TServerSettings Settings);


//=============================================================================
// ClientSendServerSettings
//
// Called on the client by the server to replicate server settings.
//=============================================================================

simulated function ClientSendServerSettings(TournamentUtil.TServerSettings Settings)
{
  if ( ViewPort(Player) == None || !PlayerReplicationInfo.bAdmin && Level.NetMode != NM_StandAlone ) {
    log("Unwanted server settings received for"@Self);
    return;
  }
  
  OnClientReceivedSettings(Settings);
}


//=============================================================================
// ServerRequestServerSettings
//
// Called on the server by the client's admin config panel to send server
// settings to the client.
//=============================================================================

function ServerRequestServerSettings()
{
  local TournamentUtil.TServerSettings Settings;
  
  if ( !PlayerReplicationInfo.bAdmin && Level.NetMode != NM_StandAlone ) {
    log(Self@"tried to request server settings");
    return;
  }
  
  Settings.SpawnProtectionTime   = DeathMatch(Level.Game).SpawnProtectionTime;
  Settings.MinPlayerNameLength   = class'TournamentServerSettings'.default.MinNameLength;
  Settings.WarmupTime            = class'TournamentServerSettings'.default.WarmupTime;
  Settings.MaxFOV                = class'TournamentServerSettings'.default.MaxFOV;
  Settings.SkinGlow              = class'TournamentServerSettings'.default.SkinGlow;
  Settings.ServerDemoName        = Left(class'TournamentServerSettings'.default.ServerDemoName, 50);
  class'TournamentUtil'.static.CompressBools(Settings.WeaponConfigBools, 3,
      class'TournamentServerSettings'.default.bWeaponModifications,
      Level.Game.bWeaponShouldViewShake,
      class'TournamentServerSettings'.default.bDisableLockdown
  );
  class'TournamentUtil'.static.CompressBools(Settings.TournamentConfigBools, 7,
      class'TournamentServerSettings'.default.bEnableNickFiltering,
      class'TournamentServerSettings'.default.bAllowHitSounds,
      class'TournamentServerSettings'.default.bAllowVariableHitSounds,
      class'TournamentServerSettings'.default.bAutoServerDemo,
      class'TournamentServerSettings'.default.bBroadcastUDamageKills,
      class'TournamentServerSettings'.default.bNoHudClock,
      class'TournamentServerSettings'.default.bAllowCarrierTelefrags
  );
  class'TournamentUtil'.static.CompressBools(Settings.AdrenalineWarmupBools, 8,
      class'TournamentServerSettings'.default.bWarmupWeapons,
      class'TournamentServerSettings'.default.bWarmupAmmo,
      class'TournamentServerSettings'.default.bWarmupHealth,
      class'TournamentServerSettings'.default.bWarmupShield,
      class'TournamentServerSettings'.default.bAllowComboBerserk,
      class'TournamentServerSettings'.default.bAllowComboBooster,
      class'TournamentServerSettings'.default.bAllowComboInvisible,
      class'TournamentServerSettings'.default.bAllowComboSpeed
  );
  
  // send values of server-side properties
  ClientSendServerSettings(Settings);
}


//=============================================================================
// ServerSetServerSettings
//
// Called on the server by the client's admin config panel to send the new
// settings to the server.
//=============================================================================

function ServerSetServerSettings(TournamentUtil.TServerSettings Settings)
{
  local Controller thisController;
  local TournamentPawn ThisPawn;
  local TournamentGameReplicationInfo TGRI;
  
  if ( !PlayerReplicationInfo.bAdmin && Level.NetMode != NM_StandAlone ) {
    log(Self@"tried to apply server settings");
    return;
  }
  
  DeathMatch(Level.Game).SpawnProtectionTime = Settings.SpawnProtectionTime;
  class'TournamentServerSettings'.default.MinNameLength  = Settings.MinPlayerNameLength;
  class'TournamentServerSettings'.default.WarmupTime     = Settings.WarmupTime;
  class'TournamentServerSettings'.default.MaxFOV         = Settings.MaxFOV;
  class'TournamentServerSettings'.default.SkinGlow       = Settings.SkinGlow;
  class'TournamentServerSettings'.default.ServerDemoName = Settings.ServerDemoName;
  
  class'TournamentServerSettings'.default.bWeaponModifications
      = class'TournamentUtil'.static.GetCompressedBool(Settings.WeaponConfigBools, 0);
  Level.Game.bWeaponShouldViewShake
      = class'TournamentUtil'.static.GetCompressedBool(Settings.WeaponConfigBools, 1);
  class'TournamentServerSettings'.default.bDisableLockdown
      = class'TournamentUtil'.static.GetCompressedBool(Settings.WeaponConfigBools, 2);

  class'TournamentServerSettings'.default.bEnableNickFiltering
      = class'TournamentUtil'.static.GetCompressedBool(Settings.TournamentConfigBools, 0);
  class'TournamentServerSettings'.default.bAllowHitSounds
      = class'TournamentUtil'.static.GetCompressedBool(Settings.TournamentConfigBools, 1);
  class'TournamentServerSettings'.default.bAllowVariableHitSounds
      = class'TournamentUtil'.static.GetCompressedBool(Settings.TournamentConfigBools, 2);
  class'TournamentServerSettings'.default.bAutoServerDemo
      = class'TournamentUtil'.static.GetCompressedBool(Settings.TournamentConfigBools, 3);
  class'TournamentServerSettings'.default.bBroadcastUDamageKills
      = class'TournamentUtil'.static.GetCompressedBool(Settings.TournamentConfigBools, 4);
  class'TournamentServerSettings'.default.bNoHudClock
      = class'TournamentUtil'.static.GetCompressedBool(Settings.TournamentConfigBools, 5);
  class'TournamentServerSettings'.default.bAllowCarrierTelefrags
      = class'TournamentUtil'.static.GetCompressedBool(Settings.TournamentConfigBools, 6);
  
  class'TournamentServerSettings'.default.bWarmupWeapons
      = class'TournamentUtil'.static.GetCompressedBool(Settings.AdrenalineWarmupBools, 0);
  class'TournamentServerSettings'.default.bWarmupAmmo
      = class'TournamentUtil'.static.GetCompressedBool(Settings.AdrenalineWarmupBools, 1);
  class'TournamentServerSettings'.default.bWarmupHealth
      = class'TournamentUtil'.static.GetCompressedBool(Settings.AdrenalineWarmupBools, 2);
  class'TournamentServerSettings'.default.bWarmupShield
      = class'TournamentUtil'.static.GetCompressedBool(Settings.AdrenalineWarmupBools, 3);
  
  class'TournamentServerSettings'.default.bAllowComboBerserk
      = class'TournamentUtil'.static.GetCompressedBool(Settings.AdrenalineWarmupBools, 4);
  class'TournamentServerSettings'.default.bAllowComboBooster
      = class'TournamentUtil'.static.GetCompressedBool(Settings.AdrenalineWarmupBools, 5);
  class'TournamentServerSettings'.default.bAllowComboInvisible
      = class'TournamentUtil'.static.GetCompressedBool(Settings.AdrenalineWarmupBools, 6);
  class'TournamentServerSettings'.default.bAllowComboSpeed
      = class'TournamentUtil'.static.GetCompressedBool(Settings.AdrenalineWarmupBools, 7);
  
  Level.Game.SaveConfig();
  class'TournamentServerSettings'.static.StaticSaveConfig();
  
  TGRI = TournamentGameReplicationInfo(GameReplicationInfo);
  if ( TGRI != None ) {
    TGRI.SpawnProtectionTime  = DeathMatch(Level.Game).SpawnProtectionTime;
    TGRI.MaxFOV               = class'TournamentServerSettings'.default.MaxFOV;
    TGRI.bNoHudClock          = class'TournamentServerSettings'.default.bNoHudClock;
    TGRI.bWeaponModifications = class'TournamentServerSettings'.default.bWeaponModifications;
  }
  
  foreach DynamicActors(class'TournamentPawn', ThisPawn)
    if ( ThisPawn.Health > 0 )
      ThisPawn.AmbientGlow = class'TournamentServerSettings'.default.SkinGlow;
  
  // confirm new values of submitted properties
  ServerRequestServerSettings();
  
  for (thisController = Level.ControllerList; thisController != None; thisController = thisController.nextController)
    if ( thisController.IsA('PlayerController') )
      Level.Game.SetWeaponViewShake(PlayerController(thisController));
}
