//=============================================================================
// TournamentPlayerReplicationInfo
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// PRI class for Tournament games.
//=============================================================================


class TournamentPlayerReplicationInfo extends xPlayerReplicationInfo;


//=============================================================================
// Structs
//=============================================================================

struct TScoreEvent {
  var() string Description;             // the description of the score event
  var() int Count;                      // how often this score event happened
};

struct TPickupStats {
  var() class<Pickup> Pickup;
  var() bool bSlotUsed;                 // whether there's actually something in this slot
  var() int Count;                      // how often this type of item was picked up
  var() int Amount;                     // how much shield, health or adrenaline was gained
};

struct TPRIData {
  var() float Score;
  var() float Deaths;
  var() int NumLives;
  var() string PlayerName;
  var() bool bOutOfLives;
  var() int SuicideCount;
  var() int KillCount;
  var() int Goals;
};


//=============================================================================
// Variables
//=============================================================================

var private TScoreEvent ScoreEvents[100];
var() private TPickupStats PickupStats[16];
var PlayerReplicationInfo RealNameCamper; // who camps my real nick?
var string RealName;
var string PrevName;
var bool bInitialized;

var int TouchCount;
var int ReturnCount;
var int SuicideCount;
var int KillCount;
var int Goals;
var int Rank;

var int PacketLoss;
var int NetSpeed;


//=============================================================================
// Replication
//=============================================================================

replication
{
  reliable if ( Role == ROLE_Authority )
    ScoreEvents, SuicideCount, KillCount, Goals, TouchCount, ReturnCount,
    PacketLoss, NetSpeed, PickupStats;
}


//=============================================================================
// GetPlayerData
//
// Returns a TPRIData struct containing the important replicated PRI values.
//=============================================================================

function TPRIData GetPlayerData()
{
  local TPRIData PlayerData;
  
  PlayerData.Score        = Score;
  PlayerData.Deaths       = Deaths;
  PlayerData.NumLives     = NumLives;
  PlayerData.PlayerName   = PlayerName;
  PlayerData.bOutOfLives  = bOutOfLives;
  PlayerData.SuicideCount = SuicideCount;
  PlayerData.KillCount    = KillCount;
  PlayerData.Goals        = Goals;
  
  return PlayerData;
}


//=============================================================================
// SetPlayerData
//
// Sets the important replicated PRI values from a TPRIData struct.
//=============================================================================

function SetPlayerData(TPRIData PlayerData)
{
  Score        = PlayerData.Score;
  Deaths       = PlayerData.Deaths;
  NumLives     = PlayerData.NumLives;
  PlayerName   = PlayerData.PlayerName;
  bOutOfLives  = PlayerData.bOutOfLives;
  SuicideCount = PlayerData.SuicideCount;
  KillCount    = PlayerData.KillCount;
  Goals        = PlayerData.Goals;
}


//=============================================================================
// CountPickup
//
// Counts the number of pickups and the amount of health/shield/adrenaline.
//=============================================================================

function CountPickup(TournamentPawn Other, class<Pickup> PickupClass)
{
  local int Amount;
  local int PickupIndex;
  
  if ( Other == None || class<TournamentPickup>(PickupClass) == None )
    return;
  
  PickupIndex = GetPickupIndex(PickupClass);
  if ( PickupIndex == ArrayCount(PickupStats) )
    return;
  
  if ( class<TournamentHealth>(PickupClass) != None )
    Amount = Other.LastHealthAdd;
  else if ( class<ShieldPickup>(PickupClass) != None )
    Amount = Other.LastShieldAdd;
  else if ( class<AdrenalinePickup>(PickupClass) != None && Other.Controller != None ) {
    if ( Other.Controller != None )
      Amount = Other.Controller.Adrenaline - Other.LastAdrenalineAmount;
    else if ( Controller(Other.Owner) != None )
      Amount = Controller(Other.Owner).Adrenaline - Other.LastAdrenalineAmount;
  }
  
  PickupStats[PickupIndex].Count++;
  PickupStats[PickupIndex].Amount += Amount;
  PickupStats[PickupIndex] = PickupStats[PickupIndex];
}


//=============================================================================
// NumSlots
//
// Returns the number of stats slots used.
//=============================================================================

simulated function int NumSlots()
{
  local int i;
  
  // count slots
  while (i < ArrayCount(PickupStats) && PickupStats[i].bSlotUsed)
    i++;
  
  return i;
}


//=============================================================================
// SlotUsed
//
// Returns whether the specified slot is actually used.
//=============================================================================

simulated function bool SlotUsed(int Slot)
{
  if ( Slot != Clamp(Slot, 0, ArrayCount(PickupStats)) )
    return false;
  
  return PickupStats[Slot].bSlotUsed;
}


//=============================================================================
// GetStatsSlot
//
// Returns the specified slot's stats.
//=============================================================================

simulated function TPickupStats GetStatsSlot(int Slot)
{
  return PickupStats[Clamp(Slot, 0, ArrayCount(PickupStats))];
}


//=============================================================================
// GetPickupIndex
//
// Returns the index of a pickup class in the PickupStats array.
//=============================================================================

function int GetPickupIndex(class<Pickup> Pickup)
{
  local int i;
  
  // find the pickup class
  for (i = 0; i < ArrayCount(PickupStats) && PickupStats[i].bSlotUsed; i++)
    if ( PickupStats[i].Pickup == Pickup )
      return i;
  
  if ( i < ArrayCount(PickupStats) ) { // nothing found, add the pickup class if there are still slots left
    PickupStats[i].bSlotUsed = True;
    PickupStats[i].Pickup = Pickup;
    SortArray(0, i);
    return GetPickupIndex(Pickup);
  }
  
  return i;
}


//=============================================================================
// SortArray
//
// Sorts the PickupStats array using QuickSort.
// http://wiki.beyondunreal.com/wiki/QuickSort
//=============================================================================

function SortArray(int Low, int High)
{
  local int i, j;
  local TPickupStats x;

  i = Low;
  j = High;
  x = PickupStats[(Low + High) / 2];

  //  partition
  do {  
    while (PickupStats[i] < x)
      i += 1; 
    while (PickupStats[j] > x)
      j -= 1;
    if ( i <= j ) {
      SwapArray(i, j);
      i += 1; 
      j -= 1;
    }
  } until (i > j);

  //  recursion
  if ( low < j )
    SortArray(low, j);
  if ( i < high )
    SortArray(i, high);
}


//=============================================================================
// SwapArray
//
// Swaps the specified elements in the PickupStats array.
// http://wiki.beyondunreal.com/wiki/QuickSort
//=============================================================================

function SwapArray(int EL1, int EL2)
{
  local TPickupStats AZ1;
  local TPickupStats AZ2;

  AZ2 = PickupStats[EL1];
  AZ1 = PickupStats[EL2];

  PickupStats[EL2] = AZ2;
  PickupStats[EL1] = AZ1;
}


//=============================================================================
// operator >
// operator <
// GetRating
//
// Greater than and less then operators for TWeaponStats structs used in sort.
//=============================================================================

static final operator(24) bool > (TPickupStats A, TPickupStats B)
{
  if ( !A.bSlotUsed ) {
    return !B.bSlotUsed;
  }
  else {
    if ( !B.bSlotUsed )
      return false;
    else {
      return GetRating(A.Pickup) > GetRating(B.Pickup);
    }
  }
}

static final operator(24) bool < (TPickupStats A, TPickupStats B)
{
  return B > A;
}

static function int GetRating(class<Pickup> PClass)
{
  if ( class<TournamentHealth>(PClass) != None )
    return 1000 + class<TournamentHealth>(PClass).default.HealingAmount;
  else if ( class<ShieldPickup>(PClass) != None )
    return 2000 + class<ShieldPickup>(PClass).default.ShieldAmount;
  else if ( class<AdrenalinePickup>(PClass) != None )
    return 3000 + class<AdrenalinePickup>(PClass).default.AdrenalineAmount;
  else if ( class<UDamagePack>(PClass) != None )
    return 4000;
  else
    return 5000;
}


//=============================================================================
// PostNetReceive
//
// Send a message when a player changed the name.
//=============================================================================

simulated event PostNetReceive()
{
  local PlayerController LocalPlayer;
  
  LocalPlayer = Level.GetLocalPlayerController();
  if ( LocalPlayer != None && PrevName != "" && PrevName != PlayerName ) {
    OldName = PrevName;
    LocalPlayer.ReceiveLocalizedMessage(Level.GetGameClass().default.GameMessageClass, 2, Self);
  }
  PrevName = PlayerName;
}


//=============================================================================
// SetRealName
//
// Save the real name in case it becomes available again.
//=============================================================================

function SetRealName(string S)
{
  if ( !bBot )
    RealName = S;
}


//=============================================================================
// RealNameFreed
//
// The real name became available.
//=============================================================================

function RealNameFreed()
{
  RealNameCamper = None;
  if ( RealName != "" && RealName != PlayerName )
    Level.Game.ChangeName(Controller(Owner), RealName, True);
}


//=============================================================================
// SetPlayerName
//
// Sets the player name.
//=============================================================================

function SetPlayerName(string S)
{
  Super.SetPlayerName(S);
  if ( OldName == "" )
    OldName = S;
  if ( RealName == S )
    RealNameCamper = None;
  
  if ( bInitialized )
    PostNetReceive();
  bInitialized = true;
}


//=============================================================================
// Timer
//
// Check if the real name became available.
//=============================================================================

function Timer()
{
  Super.Timer();
  
  KillCount = Kills;
  Goals = GoalsScored;
  TouchCount = FlagTouches;
  ReturnCount = FlagReturns;
  if ( !bBot && Level.NetMode != NM_StandAlone && PlayerController(Owner) != None
      && PlayerController(Owner).Player != None ) {
    PacketLoss = int(Owner.ConsoleCommand("GETLOSS"));
    NetSpeed   = PlayerController(Owner).Player.CurrentNetSpeed;
  }
  if ( RealName != "" && RealName != PlayerName && (RealNameCamper == None || RealNameCamper.PlayerName != RealName) )
    RealNameFreed();
  
  SetTimer(0.5 + 0.5 * FRand(), true);
}


//=============================================================================
// AddScoreEvent
//
// Increase the count for a score event. The score event will be added to the
// list if it doesn't already exist.
//=============================================================================

function AddScoreEvent(string Description)
{
  local int i;
  
  for (i = 0; i < ArrayCount(ScoreEvents); i++) {
    if ( ScoreEvents[i].Description == "" )
      ScoreEvents[i].Description = Description;
    if ( ScoreEvents[i].Description ~= Description ) {
      ScoreEvents[i].Count++;
      ScoreEvents[i] = ScoreEvents[i];
      return;
    }
  }
}


//=============================================================================
// GetEventCount
//
// Returns, how often a score event happened.
//=============================================================================

simulated function int GetEventCount(string Description)
{
  local int i;
  
  for (i = 0; i < ArrayCount(ScoreEvents); i++)
    if ( ScoreEvents[i].Description ~= Description )
      return ScoreEvents[i].Count;
  
  return 0;
}


//=============================================================================
// Reset
//
// Reset the PRI after warmup.
//=============================================================================

function Reset()
{
  local int i;
  local TScoreEvent EmptyEvent;
  local TPickupStats EmptyStats;
  
  if ( bOnlySpectator )
    return;
  
  for (i = 0; i < ArrayCount(ScoreEvents); i++)
    ScoreEvents[i] = EmptyEvent;
  for (i = 0; i < ArrayCount(PickupStats); i++)
    PickupStats[i] = EmptyStats;
  for (i = 0; i < ArrayCount(Spree); i++)
    Spree[i] = 0;
  for (i = 0; i < ArrayCount(MultiKills); i++)
    MultiKills[i] = 0;
  for (i = 0; i < ArrayCount(Combos); i++)
    Combos[i] = 0;
  
  Suicides = 0;
  Kills = 0;
  StartTime = 0;
  bFirstBlood = False;
  WeaponStatsArray.Length = 0;
  FlagTouches = 0;
  FlagReturns = 0;
  GoalsScored = 0;
  Suicides = 0;
  flakcount = 0;
  combocount = 0;
  headcount = 0;
  SuicideCount = 0;
  KillCount = 0;
  Goals = 0;
  TouchCount = 0;
  ReturnCount = 0;
  
  Super.Reset();
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  bNetNotify=True
}