//=============================================================================
// TournamentUtil
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Contains utility functions used by several classes to reduce duplicate code.
//=============================================================================


class TournamentUtil extends Info
    abstract
    exportstructs;


//=============================================================================
// Structs
//=============================================================================

struct TServerSettings {
  var int    SpawnProtectionTime;
  var int    MinPlayerNameLength;
  var int    WarmupTime;
  var int    MaxFOV;
  var byte   SkinGlow;
  var byte   TournamentConfigBools;
  var byte   AdrenalineWarmupBools;
  var byte   WeaponConfigBools;
  var string ServerDemoName;
};


//=============================================================================
// Localization
//=============================================================================

var localized string TournamentBuild;

var localized array<string> DayName;
var localized array<string> MonthName;
var localized string DateTimeString;

var localized string AdrenalineString;
var localized string HealthVialString;
var localized string HealthPackString;
var localized string SuperHealthString;
var localized string ShieldPackString;
var localized string SuperShieldString;
var localized string UDamageString;


//=============================================================================
// Variables
//=============================================================================

var() const editconst string Build;

var string ValidNameChars;
var string NameMustContain;

var Sound CountDown[10];
var Sound LongCount[6];


//=============================================================================
// GetTournamentVersion
//
// Returns the Tournament build.
//=============================================================================

static function string GetTournamentVersion()
{
  return default.TournamentBuild@default.Build;
}


//=============================================================================
// BinarySearch
//
// Search a string array for an element, return its index.
// Modified from wUtils.wArray (http://wiki.beyondunreal.com/WUtils)
//=============================================================================

static final function int BinarySearch(array<string> MyArray, string SearchString, optional bool bReturnClosest)
{
  local int Middle, Low, High;
  
  High = MyArray.Length - 1;
  SearchString = Caps(SearchString);
  
  while (Low <= High) {
    Middle = (Low + High) / 2;
    if ( MyArray[Middle] ~= SearchString )
      return Middle;
    if (Caps( MyArray[Middle]) > SearchString )
      High = Middle - 1;
    else if ( Caps(MyArray[Middle]) < SearchString )
      Low = Middle + 1;
  }
  if ( bReturnClosest ) {
    if ( MyArray.Length > 0 && Low < MyArray.Length && Caps(MyArray[Low]) < SearchString )
      Low++;
    return Low;
  }
  return -1;           
}


//=============================================================================
// SpawnBot
//
// Use the SuperBots bot controller class if desired.
//=============================================================================

static function Bot SpawnBot(DeathMatch Game, optional string BotName)
{
  
  local Bot NewBot;
  local RosterEntry Chosen;
  local UnrealTeamInfo BotTeam;
  
  BotTeam = Game.GetBotTeam();
  Chosen = BotTeam.ChooseBotClass(BotName);
  
  if ( Chosen.PawnClass == None ) {
    if ( Chosen.PawnClassName ~= "xGame.xPawn" )
      Chosen.PawnClassName = Game.DefaultPlayerClassName;
    Chosen.Init();
  }
  else if ( Chosen.PawnClass == class'xPawn' )
    Chosen.PawnClass = class'TournamentPawn';
  
  NewBot = Bot(Game.Spawn(Chosen.PawnClass.default.ControllerClass));
  
  if ( NewBot != None ) {
    Game.InitializeBot(NewBot, BotTeam, Chosen);
    if ( NewBot.PawnClass == class'xPawn' )
      NewBot.PawnClass = class'TournamentPawn';
  }
  return NewBot;
}


//=============================================================================
// CheckWelcome
//
// If we are a server, broadcast a welcome message.
//=============================================================================

static function CheckWelcome(DeathMatch Game)
{
  local int i;
  
  if ( Game.bWelcomePending ) {
    Game.bWelcomePending = false;
    if ( Game.Level.NetMode != NM_StandAlone )
      for (i = 0; i < Game.GameReplicationInfo.PRIArray.Length; i++)
        if ( Game.GameReplicationInfo.PRIArray[i] != None && !Game.GameReplicationInfo.PRIArray[i].bWelcomed ) {
          Game.GameReplicationInfo.PRIArray[i].bWelcomed = true;
          if ( !Game.GameReplicationInfo.PRIArray[i].bOnlySpectator )
            Game.BroadcastLocalizedMessage(Game.GameMessageClass, 1, Game.GameReplicationInfo.PRIArray[i]);
          else  // welcome new spectator
            Game.BroadcastLocalizedMessage(Game.GameMessageClass, 12, Game.GameReplicationInfo.PRIArray[i]);
        }
  }
}


//=============================================================================
// PlayAnnouncement
//
// End warmup and reset the map for the real match.
//=============================================================================

static function PlayAnnouncement(DeathMatch Game, Sound Announcement, byte AnnouncementLevel, optional bool bForce)
{
  local Controller P;
  
  for (P = Game.Level.ControllerList; P != None; P = P.NextController)
    if ( P.IsA('UnrealPlayer') )
      UnrealPlayer(P).PlayAnnouncement(Announcement, AnnouncementLevel, bForce);
}


//=============================================================================
// PendingMatchTimer
//
// Check for match or warmup start.
//=============================================================================

static function PendingMatchTimer(DeathMatch Game, int WarmupTime, out int WarmupCountDown)
{
  local Controller P;
  local bool bReady;
  
  // first check if there are enough net players, and enough time has elapsed to give people
  // a chance to join
  if ( Game.NumPlayers == 0 )
    Game.bWaitForNetPlayers = true;
  if ( Game.bWaitForNetPlayers && Game.Level.NetMode != NM_Standalone ) {
    if ( Game.NumPlayers > 0 )
      Game.ElapsedTime++;
    else
      Game.ElapsedTime = 0;
    if ( Game.NumPlayers == Game.MaxPlayers || Game.ElapsedTime > Game.NetWait
        && Game.NumPlayers >= Game.MinNetPlayers ) {
      Game.bWaitForNetPlayers = false;
      Game.CountDown = Game.Default.CountDown;
      WarmupCountDown = WarmupTime;
    }
  }
  
  if ( Game.Level.NetMode != NM_Standalone && (Game.bWaitForNetPlayers || Game.bTournament
      && Game.NumPlayers < Game.MaxPlayers) ) {
    WarmupCountDown = WarmupTime;
    Game.StartupStage = 0;
    Game.bStartedCountDown = false;
    Game.PlayStartupMessage();
    return;
  }
  
  // check if players are ready
  bReady = true;
  Game.StartupStage = 1;
  WarmupCountdown--;
  if ( (WarmupCountdown > 3 || Game.Level.NetMode == NM_Standalone) && !Game.bStartedCountDown
      && (Game.bTournament || Game.bPlayersMustBeReady || Game.Level.NetMode == NM_Standalone) ) {
    for (P = Game.Level.ControllerList; P != None; P = P.NextController)
      if ( P.IsA('PlayerController') && P.PlayerReplicationInfo != None && P.bIsPlayer
          && !P.PlayerReplicationInfo.bOnlySpectator && !P.PlayerReplicationInfo.bReadyToPlay )
        bReady = false;
  }
  if ( bReady ) {
    Game.bStartedCountDown = true;
    Game.CountDown--;
    if ( Game.CountDown <= 0 ) {
      EndWarmup(Game, WarmupCountdown);
      TournamentGameReplicationInfo(Game.GameReplicationInfo).GameStartTime = Game.GameReplicationInfo.ElapsedTime;
      Game.StartMatch();
    }
    else {
      Game.StartupStage = 5 - Game.CountDown;
      if ( Game.Level.NetMode != NM_Standalone && (Game.bTournament || Game.bPlayersMustBeReady)
          && Game.CountDown <= 10 && Game.CountDown > 0 )
        PlayAnnouncement(Game, default.CountDown[Game.CountDown - 1], 1, true);
    }
  }
  else {
    switch (WarmupCountdown) {
      case 300:
        PlayAnnouncement(Game, default.LongCount[0], 1, true);
        break;
      case 180:
        PlayAnnouncement(Game, default.LongCount[1], 1, true);
        break;
      case 120:
        PlayAnnouncement(Game, default.LongCount[2], 1, true);
        break;
      case 60:
        PlayAnnouncement(Game, default.LongCount[3], 1, true);
        break;
      case 30:
        PlayAnnouncement(Game, default.LongCount[4], 1, true);
        break;
      case 20:
        PlayAnnouncement(Game, default.LongCount[5], 1, true);
        break;
      default:
        if ( WarmupCountdown <= 10 && WarmupCountdown > 0 )
          PlayAnnouncement(Game, default.CountDown[WarmupCountdown - 1], 1, true);
    }
  }
  Game.PlayStartupMessage();
}


//=============================================================================
// EndWarmup
//
// End warmup and reset the map for the real match.
//=============================================================================

static function EndWarmup(DeathMatch Game, out int RemainingWarmupTime)
{
  local Actor A;
  local xPickupBase PB;
  local GameObject GO;
  local Projectile Proj;
  
  if ( Game.bStartedCountdown && Game.CountDown > 0 )
    return;
  
  Game.bStartedCountDown = true;
  
  foreach Game.AllActors(class'GameObject', GO) {
    if ( GO.bHeld )
      GO.Drop(vect(0,0,0));
    GO.SendHome();
  }
  
  foreach Game.AllActors(class'Actor', A)
    if ( LevelInfo(A) == None && A != Game && (PlayerController(A) == None
        || PlayerController(A).PlayerReplicationInfo != None
        && !PlayerController(A).PlayerReplicationInfo.bOnlySpectator) )
      A.Reset();
  
  foreach Game.DynamicActors(class'Projectile', Proj)
    Proj.Destroy();
  
  foreach Game.AllActors(class'xPickupBase', PB)
    if ( PB.bDelayedSpawn && PB.myPickup != None && PB.myPickup.IsInState('Pickup') )
      PB.myPickup.GotoState('Sleeping');
}


//=============================================================================
// ChangeName
//
// Sets a player's (new) name.
//=============================================================================

static function ChangeName(Controller Other, string NewName, bool bNameChange)
{
  local Controller C;
  local bool bIsValidName;
  
  if ( NewName == "" || Other == None )
    return;
  
  NewName = Left(NewName, 20);
  Other.ReplaceText(NewName, " ", "_");
  
  bIsValidName = IsValidName(NewName, Other);
  
  if ( bNameChange && !bIsValidName )
    return;
  else if ( !bIsValidName )
    NewName = Other.Level.Game.DefaultPlayerName;
  
  if ( TournamentPlayerReplicationInfo(Other.PlayerReplicationInfo) != None )
    TournamentPlayerReplicationInfo(Other.PlayerReplicationInfo).SetRealName(NewName);
  
  if ( Other.PlayerReplicationInfo.PlayerName ~= NewName )
    return;
  
  if ( DeathMatch(Other.Level.Game).NamePrefixes[3] == DeathMatch(Other.Level.Game).NamePrefixes[7]
      && DeathMatch(Other.Level.Game).NameSuffixes[3] == DeathMatch(Other.Level.Game).NameSuffixes[7] )
    DeathMatch(Other.Level.Game).NamePrefixes[7] = "ber";
  
  if ( DeathMatch(Other.Level.Game).bEpicNames && Bot(Other) != None ) {
    if ( DeathMatch(Other.Level.Game).TotalEpic < 21 ) {
      NewName = DeathMatch(Other.Level.Game).EpicNames[DeathMatch(Other.Level.Game).EpicOffset % 21];
      DeathMatch(Other.Level.Game).EpicOffset++;
      DeathMatch(Other.Level.Game).TotalEpic++;
    }
    else {
      NewName = DeathMatch(Other.Level.Game).NamePrefixes[DeathMatch(Other.Level.Game).NameNumber % 10]
          $ DeathMatch(Other.Level.Game).EpicNames[DeathMatch(Other.Level.Game).EpicOffset % 21]
          $ DeathMatch(Other.Level.Game).NameSuffixes[DeathMatch(Other.Level.Game).NameNumber % 10];
      DeathMatch(Other.Level.Game).EpicOffset++;
      DeathMatch(Other.Level.Game).NameNumber++;
    }
  }
  
  for(C = Other.Level.ControllerList; C != None; C = C.nextController)
    if ( C.bIsPlayer && C.PlayerReplicationInfo.PlayerName ~= NewName ) {
      if ( Other.IsA('PlayerController') && bNameChange ) {
        PlayerController(Other).ReceiveLocalizedMessage(DeathMatch(Other.Level.Game).GameMessageClass, 8);
        if ( Other.IsA('PlayerController') && TournamentPlayerReplicationInfo(Other.PlayerReplicationInfo) != None )
          TournamentPlayerReplicationInfo(Other.PlayerReplicationInfo).RealNameCamper = C.PlayerReplicationInfo;
        return;
      }
      else {
        NewName = DeathMatch(Other.Level.Game).NamePrefixes[DeathMatch(Other.Level.Game).NameNumber % 10] $ NewName 
            $ DeathMatch(Other.Level.Game).NameSuffixes[DeathMatch(Other.Level.Game).NameNumber % 10];
        DeathMatch(Other.Level.Game).NameNumber++;
        if ( Other.IsA('PlayerController') && TournamentPlayerReplicationInfo(Other.PlayerReplicationInfo) != None )
          TournamentPlayerReplicationInfo(Other.PlayerReplicationInfo).RealNameCamper = C.PlayerReplicationInfo;
        break;
      }
    }
  
  if ( bNameChange )
    DeathMatch(Other.Level.Game).GameEvent("NameChange", NewName, Other.PlayerReplicationInfo);
  
  if ( NewName ~= "CliffyB" )
    DeathMatch(Other.Level.Game).bEpicNames = true;
  Other.PlayerReplicationInfo.SetPlayerName(NewName);
  // notify local players
  if ( bNameChange )
    //Other.BroadcastLocalizedMessage(Other.Level.Game.GameMessageClass, 2, Other.PlayerReplicationInfo);
    for (C = Other.Level.ControllerList; C != None; C = C.NextController) {
      if ( TournamentPlayerReplicationInfo(C.PlayerReplicationInfo) != None
          && TournamentPlayerReplicationInfo(C.PlayerReplicationInfo).RealNameCamper == Other.PlayerReplicationInfo ) {
        TournamentPlayerReplicationInfo(C.PlayerReplicationInfo).RealNameFreed();
        break;
      }
    }
}


//=============================================================================
// IsValidName
//
// Returns whether a name is valid for the specified Controller.
//=============================================================================

static function bool IsValidName(out string PlayerName, Controller Other)
{
  local bool bNameContainsRequiredChar;
  
  if ( PlayerController(Other) == None || Other.PlayerReplicationInfo.bOnlySpectator )
    return true;
  
  bNameContainsRequiredChar = StripInvalidChars(PlayerName);
  
  return bNameContainsRequiredChar && Len(PlayerName) >= class'TournamentServerSettings'.default.MinNameLength;
}


//=============================================================================
// StripInvalidChars
//
// Removes invalid characters from the specified player name and returns
// whether the stripped name contains one of the required characters.
//=============================================================================

static function bool StripInvalidChars(out string PlayerName)
{
  local string NextChar;
  local string StrippedName;
  local bool bNameContainsRequiredChar;
  
  while (PlayerName != "") {
    NextChar = Left(PlayerName, 1);
    PlayerName = Mid(PlayerName, 1);
    if ( InStr(default.ValidNameChars, NextChar) != -1 ) {
      StrippedName = StrippedName $ NextChar;
      if ( !bNameContainsRequiredChar && InStr(default.NameMustContain, NextChar) != -1 )
        bNameContainsRequiredChar = True;
    }
  }
  PlayerName = StrippedName;
  return bNameContainsRequiredChar || !class'TournamentServerSettings'.default.bEnableNickFiltering;
}


//=============================================================================
// GetLocalizedString
//
// Returns a localized string property's value read directly from the INT file.
//=============================================================================

static function string GetLocalizedString(Object O, string PropertyName)
{
  local string s;
  
  if ( Class(O) != None )
    s = Localize(string(O.Name), PropertyName, string(O.Outer));
  else if ( O != None )
    s = Localize(string(O.Class.Name), PropertyName, string(O.Class.Outer));
  else
    s = "";
  
  if ( Left(s, 2) == "<?" && Right(s, 2) == "?>" )
    s = "";
  
  return s;
}


//=============================================================================
// LTrim
//
// Trim leading spaces
// http://wiki.beyondunreal.com/WUtils
//=============================================================================

static final function string LTrim(coerce string S)
{
  while (Left(S, 1) == " ")
    S = Right(S, Len(S) - 1);
  return S;
}


//=============================================================================
// RTrim
//
// Trim trailing spaces
// http://wiki.beyondunreal.com/WUtils
//=============================================================================

static final function string RTrim(coerce string S)
{
  while (Right(S, 1) == " ")
    S = Left(S, Len(S) - 1);
  return S;
}


//=============================================================================
// Trim
//
// Trim leading and trailing spaces
// http://wiki.beyondunreal.com/WUtils
//=============================================================================

static final function string Trim(coerce string S)
{
  return LTrim(RTrim(S));
}


//=============================================================================
// MakeColor
//
// Returns a color struct from the color components passed to thid function.
//=============================================================================

static function color MakeColor(byte R, byte G, byte B, byte A)
{
  local color C;
  
  C.R = R;
  C.G = G;
  C.B = B;
  C.A = A;
  
  return C;
}


//=============================================================================
// ParseChatPercVar
//
// The %L chat variable should also find shield and big health packs.
// Also pick up TTM's %P and %G variables.
//=============================================================================

static function string ParseChatPercVar(Mutator BaseMutator, Controller Who, string Cmd)
{
  local float minDist, currDist;
  local Actor Closest;
  local string near, where, locName;
  local xPickupBase B;
  local NavigationPoint NP;
  
  if ( cmd ~= "%L" || cmd ~= "%G" || cmd ~= "%P" ) {
    if ( Who.Pawn == None )
      return Who.PlayerReplicationInfo.StringSpectating;
    minDist = 10000000.0;
    
    // Check for a nearby game objective
    for (NP = Who.Level.NavigationPointList; NP != None; NP = NP.NextNavigationPoint) {
      if ( GameObjective(NP) != None ) {
        currDist = VSize(NP.Location - Who.Pawn.Location);
        if ( currDist < minDist ) {
          minDist = currDist;
          Closest = NP;
        }
      }
    }
    
    // If our closest gameobjective is more than 1024, look for nearby pickup base
    if ( minDist > 1024 ) // Look for closer objects
      foreach Who.AllActors(class'xPickupBase', B)
        if ( B.Region.Zone == Who.Pawn.Region.Zone || VSize(B.Location - Who.Pawn.Location) < 2048 ) {
          if ( B.Powerup != None && B.Powerup.default.InventoryType != None )
            near = B.Powerup.default.InventoryType.static.StaticItemName();
          else if ( B.Powerup != None )
            near = GetPickupName(B.Powerup);
          else
            near = "";
          
          if ( near != "" ) {
            currDist = VSize(B.Location - Who.Pawn.Location);
            if ( currDist < minDist ) {
               minDist = currDist;
               Closest = B;
            }
          }
        }
    
    if ( Who != None && Who.PlayerReplicationInfo != None )
      locName = "(" $ Who.PlayerReplicationInfo.GetLocationName() $ ")";
    
    if ( Closest != None ) {
      if ( GameObjective(Closest) != None )
        return class'TeamGame'.Default.NearString @ GameObjective(Closest).GetHumanReadableName() @ locName;
      else if ( xPickupBase(Closest) != None && xPickupBase(Closest).Powerup != None ) {
        if ( xPickupBase(Closest).Powerup.default.InventoryType != None )
          near = xPickupBase(Closest).Powerup.default.InventoryType.static.StaticItemName();
        else
          near = GetPickupName(xPickupBase(Closest).Powerup);
        where = Who.Level.Game.FindTeamDesignation(PlayerController(Who).GameReplicationInfo, Closest);
        
        if ( Where == "" )
          return class'TeamGame'.Default.NearString @ Near @ locName;
        else
          return class'TeamGame'.Default.NearString @ where $ near @ locName;
      }
    }
    return locName;
  }
  
  return class'TeamGame'.static.ParseChatPercVar(BaseMutator, Who,Cmd);
}


//=============================================================================
// GetPickupName
//
// Returns a localized string for a pickup's name.
//=============================================================================

static function string GetPickupName(class<Pickup> PickupClass)
{
  switch (PickupClass) {
    case class'AdrenalinePickup':
      return default.AdrenalineString;
    case class'MiniHealthPack':
      return default.HealthVialString;
    case class'HealthPack':
      return default.HealthPackString;
    case class'SuperHealthPack':
      return default.SuperHealthString;
    case class'ShieldPack':
      return default.ShieldPackString;
    case class'SuperShieldPack':
      return default.SuperShieldString;
    case class'UDamagePack':
      return default.UDamageString;
    default:
      if ( PickupClass.default.InventoryType != None )
        return PickupClass.default.InventoryType.static.StaticItemName();
      return string(PickupClass.Name);
  }
}


//=============================================================================
// CompressBools
//
// Compresses up to 8 bool values into a byte value.
//=============================================================================

static function CompressBools(out byte Output, byte NumValues, bool B1, optional bool B2, optional bool B3,
    optional bool B4, optional bool B5, optional bool B6, optional bool B7, optional bool B8)
{
  local byte Result;
  
  if ( B1 ) Result = Result |   1;
  if ( B2 ) Result = Result |   2;
  if ( B3 ) Result = Result |   4;
  if ( B4 ) Result = Result |   8;
  if ( B5 ) Result = Result |  16;
  if ( B6 ) Result = Result |  32;
  if ( B7 ) Result = Result |  64;
  if ( B8 ) Result = Result | 128;
  
  NumValues = Clamp(NumValues, 0, 8);
  
  Output = ((Output >> NumValues) << NumValues) | Result;
}


//=============================================================================
// GetCompressedBool
//
// Extracts one bit from a byte variable.
//=============================================================================

static function bool GetCompressedBool(byte Input, byte Bit)
{
  return bool(Input & (2 ** Bit));
}


//=============================================================================
// FloatToString
//
// Converts a float to a string representation
// http://wiki.beyondunreal.com/WUtils
//=============================================================================

static final function string FloatToString(float Value, optional int Precision)
{
  local int IntPart;
  local float FloatPart;
  local string IntString, FloatString;
  
  Precision = Max(Precision, 1);  // otherwise a simple int cast should be used
  
  if ( Value < 0 ) {
    IntString = "-";
    Value *= -1;
  }
  IntPart = int(Value);
  FloatPart = Value - IntPart;
  IntString = IntString $ string(IntPart);
  FloatString = string(int(FloatPart * 10 ** Precision));
  while (Len(FloatString) < Precision)
    FloatString = "0" $ FloatString;
  
  return IntString$"."$FloatString;
}


//=============================================================================
// CleanUpTTMLeftOvers
//
// Restore the default properties of (hopefully) all classes TTM might touched.
//=============================================================================

static function CleanUpTTMLeftOvers()
{
  class'TransTrailRed'.default.mColorRange[0]  = MakeColor(180, 10, 10, 255);
  class'TransTrailRed'.default.mColorRange[1]  = MakeColor(180, 10, 10, 255);
  class'TransTrailBlue'.default.mColorRange[0] = MakeColor(80, 150, 255, 255);
  class'TransTrailBlue'.default.mColorRange[1] = MakeColor(80, 150, 255, 255);
  
  class'TransFlareRed'.default.mColorRange[0]  = MakeColor(200, 50, 50, 255);
  class'TransFlareRed'.default.mColorRange[1]  = MakeColor(200, 50, 50, 255);
  class'TransFlareBlue'.default.mColorRange[0] = MakeColor(50, 50, 200, 255);
  class'TransFlareBlue'.default.mColorRange[1] = MakeColor(50, 50, 200, 255);
  
  class'TransDeRes'.default.mColorRange[0]      = MakeColor(235, 15, 15, 255);
  class'TransDeRes'.default.mColorRange[1]      = MakeColor(235, 15, 15, 255);
  class'TranseDeResBlue'.default.mColorRange[0] = MakeColor(15, 15, 225, 255);
  class'TranseDeResBlue'.default.mColorRange[1] = MakeColor(15, 15, 225, 255);
  
  class'TransEffectRed'.default.Skins[0]  = Material'XGameShadersB.TransB.TransRingRed';
  class'TransEffectBlue'.default.Skins[0] = Material'XGameShaders.Trans.TransRing';
  
  class'ShieldEffect3rdRed'.default.Skins[0]  = FinalBlend'XEffectMat.RedShell';
  class'ShieldEffect3rdRed'.default.Skins[1]  = FinalBlend'XEffectMat.RedShell';
  class'ShieldEffect3rdBLUE'.default.Skins[0] = FinalBlend'XEffectMat.BlueShell';
  class'ShieldEffect3rdBLUE'.default.Skins[1] = FinalBlend'XEffectMat.BlueShell';
  
  class'PlayerLight'.default.bHidden = False;
}


//=============================================================================
// StatsHurtRadius
//
// Like DelayedHurtRadius, but also returns whether a player was hit.
//=============================================================================

static function bool StatsHurtRadius(Actor thisActor, float DamageAmount, float DamageRadius,
    class<DamageType> DamageType, float Momentum, vector HitLocation, byte StatsMode)
{
  local actor Victims;
  local PlayerReplicationInfo InstigatorPRI;
  local float damageScale, dist;
  local vector dir;
  local bool bHitPlayer;
  
  if ( thisActor.bHurtEntry )
    return false;
  
  if ( Projectile(thisActor) != None && Projectile(thisActor).InstigatorController != None )
      InstigatorPRI = Projectile(thisActor).InstigatorController.PlayerReplicationInfo;
  else if ( Pawn(thisActor) != None && Pawn(thisActor).PlayerReplicationInfo != None )
      InstigatorPRI = Pawn(thisActor).Instigator.PlayerReplicationInfo;
  
  if ( InstigatorPRI == None && thisActor.Instigator != None )
    InstigatorPRI = thisActor.Instigator.PlayerReplicationInfo;
  
  thisActor.bHurtEntry = true;
  foreach thisActor.VisibleCollidingActors(class 'Actor', Victims, DamageRadius, HitLocation)
    if ( Victims != thisActor && Victims.Role == ROLE_Authority && !Victims.IsA('FluidSurfaceInfo') ) {
      dir = Victims.Location - HitLocation;
      dist = FMax(1, VSize(dir));
      dir = dir / dist; 
      damageScale = 1 - FMax(0, (dist - Victims.CollisionRadius) / DamageRadius);
      if ( Pawn(Victims) != None ) {
        if ( Projectile(thisActor) != None && thisActor.Instigator == None )
          Pawn(Victims).DelayedDamageInstigatorController = Projectile(thisActor).InstigatorController;
        else if ( TStatsIonCannon(thisActor) != None && thisActor.Instigator == None )
          Pawn(Victims).DelayedDamageInstigatorController = TStatsIonCannon(thisActor).InstigatorController;
        else if ( Pawn(thisActor) != None )
          Pawn(Victims).DelayedDamageInstigatorController = Pawn(thisActor).Controller;
        if ( Pawn(Victims).Controller != None && Pawn(Victims).Controller.bIsPlayer
            && InstigatorPRI != None && Pawn(Victims).PlayerReplicationInfo != InstigatorPRI )
          bHitPlayer = True;
      }
      if ( TournamentPawn(Victims) != None )
        TournamentPawn(Victims).NextDamageMode = StatsMode;
      Victims.TakeDamage(
        damageScale * DamageAmount,
        thisActor.Instigator, 
        Victims.Location - 0.5 * (Victims.CollisionHeight + Victims.CollisionRadius) * dir,
        (damageScale * Momentum * dir),
        DamageType
      );
    } 
  
  thisActor.bHurtEntry = false;
  return bHitPlayer;
}


//=============================================================================
// StatsDoTrace
//
// Trace instant-hit shot and track weapon stats.
//=============================================================================

static function StatsDoTrace(vector Start, rotator Dir, InstantFire WeaponFire, byte ModeNum, int GroupID)
{
  local vector X, End, HitLocation, HitNormal, RefNormal;
  local Actor Other;
  local int Damage;
  local bool bDoReflect;
  local int ReflectNum;
  
  ReflectNum = 0;
  while (true) {
    bDoReflect = false;
    
    X = vector(Dir);
    End = Start + WeaponFire.TraceRange * X;
    
    Other = WeaponFire.Trace(HitLocation, HitNormal, End, Start, true);
    
    if ( Other != None && (Other != WeaponFire.Instigator || ReflectNum > 0) ) {
      if ( WeaponFire.bReflective && Other.IsA('xPawn')
          && xPawn(Other).CheckReflect(HitLocation, RefNormal, WeaponFire.DamageMin * 0.25) ) {
        bDoReflect = true;
        HitNormal = vect(0,0,0);
      }
      else if ( !Other.bWorldGeometry ) {
        Damage = WeaponFire.DamageMin;
        if ( WeaponFire.DamageMin != WeaponFire.DamageMax && FRand() > 0.5 )
          Damage += Rand(1 + WeaponFire.DamageMax - WeaponFire.DamageMin);
        Damage *= WeaponFire.DamageAtten;
        if ( Pawn(Other) != None && Pawn(Other).Controller != None
            && Pawn(Other).Controller.bIsPlayer && Other != WeaponFire.Instigator
            || WeaponFire.DamageType == class'DamTypeShockBeam' && ShockProjectile(Other) != None )
          class'TournamentWeaponStats'.static.IncrementHits(WeaponFire.Instigator.Controller,
              WeaponFire.Weapon.Class, ModeNum, GroupID);
        if ( TournamentPawn(Other) != None )
          TournamentPawn(Other).NextDamageMode = ModeNum;
        Other.TakeDamage(Damage, WeaponFire.Instigator, HitLocation, WeaponFire.Momentum * X, WeaponFire.DamageType);
        HitNormal = vect(0,0,0);
      }
      else if ( WeaponAttachment(WeaponFire.Weapon.ThirdPersonActor) != None )
        WeaponAttachment(WeaponFire.Weapon.ThirdPersonActor).UpdateHit(Other, HitLocation, HitNormal);
    }
    else {
      HitLocation = End;
      HitNormal = vect(0,0,0);
    }
    
    WeaponFire.SpawnBeamEffect(Start, Dir, HitLocation, HitNormal, ReflectNum);
    
    if ( bDoReflect && ++ReflectNum < 4 ) {
      Start = HitLocation;
      Dir = Rotator(RefNormal);
    }
    else {
      break;
    }
  }
}


//=============================================================================
// AdjustDir
//
// Adjusts the shot direction so it's relative to the player's view, not the
// weapon's effect/projectile start offset.
//=============================================================================

static function AdjustDir(WeaponFire WeaponFire, out rotator Dir, vector Start, float MaxRange)
{
  local vector End, HitLocation, HitNormal;
  
  End = WeaponFire.Instigator.Location + WeaponFire.Instigator.EyePosition() + MaxRange * vector(Dir);
  if ( WeaponFire.Trace(HitLocation, HitNormal, End, WeaponFire.Instigator.Location
      + WeaponFire.Instigator.EyePosition(), false) != None )
    Dir = rotator(HitLocation - Start);
}


//=============================================================================
// CurrentDate
//
// Returns the current date as formatted string.
//=============================================================================

static final function string CurrentDate(LevelInfo Level)
{
  return Date(default.DateTimeString, Level.Year, Level.Month, Level.Day, Level.Hour, Level.Minute, Level.Second, Level.DayOfWeek);
}


//=============================================================================
// Date
//
// Returns a time formatted as a string.
// Modified from wUtils.wTime (http://wiki.beyondunreal.com/WUtils)
//=============================================================================

static final function string Date(string Format, optional int Year, optional int Month, optional int Day, optional int Hour, optional int Minute, optional int Second, optional int DayOfWeek)
{
  local array<string> f,t;
  f.length = 7;
  t.length = 7;
  f[0] = "yyyy";
  t[0] = Right("0000"$Year, 4);
  f[1] = "yy";
  t[1] = Right("00"$Year, 2);
  f[2] = "mm";
  t[2] = Right("00"$Month, 2);
  f[3] = "dd";
  t[3] = Right("00"$Day, 2);
  f[4] = "hh";
  t[4] = Right("00"$Hour, 2);
  f[5] = "nn";
  t[5] = Right("00"$Minute, 2);
  f[6] = "ss";
  t[6] = Right("00"$Second, 2);
  f[7] = "%D";
  t[7] = default.DayName[DayOfWeek];
  f[8] = "%M";
  t[8] = default.MonthName[Month];
  return StrReplace(Format, f, t);
}


//=============================================================================
// StrReplace
//
// StrReplace using an array with replacements
// will return the changed string, will replace all occurences unless bOnlyFirst
// From wUtils.wString (http://wiki.beyondunreal.com/WUtils)
//=============================================================================

static final function string StrReplace(coerce string target, array<string> replace, array<string> with, optional bool bOnlyFirst)
{
  local int i,j;
  local string Input;

  // cycle trough replacement list
  for (j = 0; j < replace.length; j++)
  {
    Input = target;
    target = "";
    i = InStr(input, Replace[j]);
    while(i != -1)
    { 
      target = target $ Left(Input, i) $ With[j];
      Input = Mid(Input, i + Len(Replace[j]));  
      if (bOnlyFirst) break; // only replace first occurance
      i = InStr(Input, Replace[j]);
    }
    target = target $ Input;
  }
  return target;
}


//=============================================================================
// FillPlayInfo
//
// Called directly from a Tournament gametype.
//=============================================================================

static function FillPlayInfo(PlayInfo PlayInfo)
{
  PlayInfo.AddSetting("Tournament", "bWeaponShouldViewShake", "Weapon View Shake", 0, 0, "Check");
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  AdrenalineString  = "Adrenaline"
  HealthVialString  = "Health Vial"
  HealthPackString  = "Health Pack"
  SuperHealthString = "Keg O' Health"
  ShieldPackString  = "Shield"
  SuperShieldString = "Super Shield"
  UDamageString     = "Double Damage"
  
  CountDown(0)=Sound'AnnouncerMain.One'
  CountDown(1)=Sound'AnnouncerMain.Two'
  CountDown(2)=Sound'AnnouncerMain.Three'
  CountDown(3)=Sound'AnnouncerMain.Four'
  CountDown(4)=Sound'AnnouncerMain.Five'
  CountDown(5)=Sound'AnnouncerMain.Six'
  CountDown(6)=Sound'AnnouncerMain.Seven'
  CountDown(7)=Sound'AnnouncerMain.Eight'
  CountDown(8)=Sound'AnnouncerMain.Nine'
  CountDown(9)=Sound'AnnouncerMain.Ten'
  
  LongCount(0)=Sound'AnnouncerMain.5_minute_warning'
  LongCount(1)=Sound'AnnouncerMain.3_minutes_remain'
  LongCount(2)=Sound'AnnouncerMain.2_minutes_remain'
  LongCount(3)=Sound'AnnouncerMain.1_minute_remains'
  LongCount(4)=Sound'AnnouncerMain.30_seconds_remain'
  LongCount(5)=Sound'AnnouncerMain.20_seconds'
  
  DateTimeString = "%D, %M dd yyyy, hh:nn:ss"
  DayName = ("Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday")
  MonthName = ("January","February","March","April","May","June","July","August","September","October","November","December")
  
  TournamentBuild = "Tournament build"
  Build           = "%%%%-%%-%% %%:%% (alpha)"
  ValidNameChars  = "!#$%&'()*+,-./0123456789:;<=>@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~"
  NameMustContain = "0123456789@ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
}