//=============================================================================
// TournamentWeaponStats
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Gathers statistics about damage and ShotsHit.
//=============================================================================


class TournamentWeaponStats extends Inventory;


//=============================================================================
// Structs
//=============================================================================

struct TWeaponStats {
  var() class<Weapon> Weapon;           
  var() bool bSlotUsed;                 // The arrays contain the values for
  var() int Shots[3];                   // primary, secondary and special firing
  var() int ShotsHit[3];                // mode in that order. The special firing
  var() int DamageHits[3];              // mode can be combo damage, headshots
  var() int TotalDamage[3];             // or bio goo already on the ground.
};


//=============================================================================
// Variables
//=============================================================================

var private TWeaponStats WeaponStats[32];
var PlayerReplicationInfo PRI;

var Controller Player;
var int NextID;
var array<int> IDs;


//=============================================================================
// Replication
//=============================================================================

replication
{
  reliable if ( Role == ROLE_Authority )
    WeaponStats;
  
  reliable if ( bNetInitial && Role == ROLE_Authority )
    PRI;
}


//=============================================================================
// GetStatsFor
//
// Returns a TournamentWeaponStats actor for the specified player.
//=============================================================================

static function TournamentWeaponStats GetStatsFor(Controller C)
{
  local Inventory Inv;
  local TournamentWeaponStats Stats;
  
  for (Inv = C.Inventory; Inv != None; Inv = Inv.Inventory)
    if ( TournamentWeaponStats(Inv) != None ) {
      Stats = TournamentWeaponStats(Inv);
      break;
    }
  
  if ( Stats == None ) {
    // no weapon stats found for this player, spawn it
    Stats = C.Spawn(class'TournamentWeaponStats', C);
    if ( Stats != None ) {
      Stats.Player = C;
      Stats.PRI = C.PlayerReplicationInfo;
      Stats.Inventory = C.Inventory;
      C.Inventory = Stats;
    }
  }
  
  return Stats;
}


//=============================================================================
// FindStatsFor
//
// Returns a TournamentWeaponStats actor for the specified PRI.
//=============================================================================

static function TournamentWeaponStats FindStatsFor(PlayerReplicationInfo PRI)
{
  local TournamentWeaponStats Stats;
  
  if ( PRI == None )
    return None;
  
  foreach PRI.DynamicActors(class'TournamentWeaponStats', Stats)
    if ( Stats.PRI == PRI )
      return Stats;
  
  return None;
}


//=============================================================================
// NumSlots
//
// Returns the number of stats slots used.
//=============================================================================

simulated function int NumSlots()
{
  local int i;
  
  // count slots
  while (i < ArrayCount(WeaponStats) && WeaponStats[i].bSlotUsed)
    i++;
  
  return i;
}


//=============================================================================
// SlotUsed
//
// Returns whether the specified slot is actually used.
//=============================================================================

simulated function bool SlotUsed(int Slot)
{
  if ( Slot != Clamp(Slot, 0, ArrayCount(WeaponStats)) )
    return false;
  
  return WeaponStats[Slot].bSlotUsed;
}


//=============================================================================
// GetStatsSlot
//
// Returns the specified slot's stats.
//=============================================================================

simulated function TWeaponStats GetStatsSlot(int Slot)
{
  return WeaponStats[Clamp(Slot, 0, ArrayCount(WeaponStats))];
}


//=============================================================================
// GetWeaponIndex
//
// Returns the index of a weapon class in the WeaponStats array.
//=============================================================================

function int GetWeaponIndex(class<Weapon> Weapon)
{
  local int i;
  
  // find the weapon class
  for (i = 0; i < 32 && WeaponStats[i].bSlotUsed; i++)
    if ( WeaponStats[i].Weapon == Weapon )
      return i;
  
  if ( i < 32 ) {
    WeaponStats[i].bSlotUsed = True;
    WeaponStats[i].Weapon = Weapon;
    SortArray(0, i);
    return GetWeaponIndex(Weapon);
  }
  
  return i;
}


//=============================================================================
// SortArray
//
// Sorts the WeaponStats array using QuickSort.
// http://wiki.beyondunreal.com/wiki/QuickSort
//=============================================================================

function SortArray(int Low, int High)
{
  local int i, j;
  local TWeaponStats x;

  i = Low;
  j = High;
  x = WeaponStats[(Low + High) / 2];

  //  partition
  do {  
    while (WeaponStats[i] < x)
      i += 1; 
    while (WeaponStats[j] > x)
      j -= 1;
    if ( i <= j ) {
      SwapArray(i, j);
      i += 1; 
      j -= 1;
    }
  } until (i > j);

  //  recursion
  if ( low < j )
    SortArray(low, j);
  if ( i < high )
    SortArray(i, high);
}


//=============================================================================
// SwapArray
//
// Swaps the specified elements in the WeaponStats array.
// http://wiki.beyondunreal.com/wiki/QuickSort
//=============================================================================

function SwapArray(int EL1, int EL2)
{
  local TWeaponStats AZ1;
  local TWeaponStats AZ2;

  AZ2 = WeaponStats[EL1];
  AZ1 = WeaponStats[EL2];

  WeaponStats[EL2] = AZ2;
  WeaponStats[EL1] = AZ1;
}


//=============================================================================
// operator >
// operator <
//
// Greater than and less then operators for TWeaponStats structs used in sort.
//=============================================================================

static final operator(24) bool > (TWeaponStats A, TWeaponStats B)
{
  local float GroupA, GroupB;
  
  if ( !A.bSlotUsed ) {
    return true;
  }
  else if ( A.Weapon == None ) {
    return !B.bSlotUsed;
  }
  else {
    if ( !B.bSlotUsed || B.Weapon == None )
      return true;
    else {
      GroupA = A.Weapon.default.InventoryGroup;
      if ( GroupA == 0 )
        GroupA = 9.5; // place after lightninggun, but before translocator
      GroupB = B.Weapon.default.InventoryGroup;
      if ( GroupB == 0 )
        GroupB = 9.5;
      if ( GroupA != GroupB )
        return GroupA > GroupB;
      else
        return A.Weapon.default.GroupOffset > B.Weapon.default.GroupOffset;
    }
  }
}

static final operator(24) bool < (TWeaponStats A, TWeaponStats B)
{
  return B > A;
}


//=============================================================================
// IncrementShots
//
// Increments the number of shots with a certain weapon for a player.
//=============================================================================

static function int IncrementShots(Controller C, class<Weapon> Weapon, byte Mode)
{
  local TournamentWeaponStats Stats;
  
  Stats = GetStatsFor(C);
  if ( Stats != None )
    Stats.IncShots(Weapon, Mode);
  
  return ++default.NextID;
}


//=============================================================================
// IncShots
//
// Increments the number of shots with a certain weapon for a player.
//=============================================================================

function IncShots(class<Weapon> Weapon, byte Mode)
{
  local int WeaponIndex;
  
  WeaponIndex = GetWeaponIndex(Weapon);
  WeaponStats[WeaponIndex].Shots[Mode]++;
  WeaponStats[WeaponIndex] = WeaponStats[WeaponIndex];
}


//=============================================================================
// IncrementHits
//
// Increments the number of ShotsHit with a certain weapon for a player.
//=============================================================================

static function IncrementHits(Controller C, class<Weapon> Weapon, byte Mode, int ProjectileID)
{
  local TournamentWeaponStats Stats;
  
  Stats = GetStatsFor(C);
  if ( Stats != None )
    Stats.IncHits(Weapon, Mode, ProjectileID);
}


//=============================================================================
// IncHits
//
// Increments the number of shots with a certain weapon for a player.
//=============================================================================

function IncHits(class<Weapon> Weapon, byte Mode, int ProjectileID)
{
  local int i;
  local int WeaponIndex;
  
  WeaponIndex = GetWeaponIndex(Weapon);
  
  if ( ProjectileID > 0 ) {
    for(i = 0; i < IDs.Length; i++)
      if ( IDs[i] == ProjectileID )
        return;
    IDs.Insert(0, 1);
    IDs[0] = ProjectileID;
    if ( IDs.Length > 32 )
      IDs.Length = 32;
  }
  WeaponStats[WeaponIndex].ShotsHit[Mode]++;
  WeaponStats[WeaponIndex] = WeaponStats[WeaponIndex];
}


//=============================================================================
// IncrementDamage
//
// Increments the number of ShotsHit with a certain weapon for a player.
//=============================================================================

static function IncrementDamage(Controller C, class<Weapon> Weapon, int Damage, byte Mode)
{
  local TournamentWeaponStats Stats;
  
  if ( Damage > 0 )
    Stats = GetStatsFor(C);
  if ( Stats != None )
    Stats.IncDamage(Weapon, Damage, Mode);
}


//=============================================================================
// IncDamage
//
// Increments the number of shots with a certain weapon for a player.
//=============================================================================

function IncDamage(class<Weapon> Weapon, int Damage, byte Mode)
{
  local int WeaponIndex;
  
  WeaponIndex = GetWeaponIndex(Weapon);
  WeaponStats[WeaponIndex].DamageHits[Mode]++;
  WeaponStats[WeaponIndex].TotalDamage[Mode] += Damage;
  WeaponStats[WeaponIndex] = WeaponStats[WeaponIndex];
}


//=============================================================================
// Destroyed
//
// Removes this actor from the Controller's Inventory list.
//=============================================================================

function Destroyed()
{
  local Inventory Inv;
  
  if ( Player != None ) {
    if ( Player.Inventory == Self )
      Player.Inventory = Inventory;
    else {
      for (Inv = Player.Inventory; Inv != None; Inv = Inv.Inventory)
        if ( Inv.Inventory == Self ) {
          Inv.Inventory = Inventory;
          break;
        }
    }
  }
  Inventory = None;
  
  Super.Destroyed();
}


//=============================================================================
// Reset
//
// Resets the weapon stats.
//=============================================================================

function Reset()
{
  Super.Reset();
  Destroy();
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  bSkipActorPropertyReplication=True
  bOnlyRelevantToOwner=False
  bOnlyDirtyReplication=False
  bAlwaysRelevant=True
  bReplicateMovement=False
}